" ------------------------------------------------------------------- "
" AnimationTags Class is a Singleton class that allows the user to    "
" reference DataType Animation-class tags' hexadecimal values.        "
""
" The User does NOT need to create one of these, since DataTypeSystem "
" Class will instantiate the only needed instance of this Class.  See "
" the DataTypeSystem.st source file for the method(s) that help the   "
" User with this Class.                                               "
""
" ALL singleton classes MUST contain the following:                   "
""
"   the methods:  isSingleton AND privateSetup     AND                "
"                 uniqueInstance Class instance variable.             "
" ------------------------------------------------------------------- "

Class AnimationTags :Dictionary ! uniqueInstance !
[
   isSingleton
     ^ true  
|  
   privateNew ! newinstance !
     newinstance <- super new.

     ^ newinstance
|
   new
     ^ self privateSetup
|
   privateInitializeDictionary

     "Animation attributes:"
     self at: #ADTA_Dummy             put: 16r80001258.

     self at: #ADTA_ModeID            put: 16r800010C8.

     "Key frame (first frame) bitmap:"
     self at: #ADTA_KeyFrame          put: 16r800010CA.

     self at: #ADTA_ColorRegisters    put: 16r800010CB.
     self at: #ADTA_CRegs             put: 16r800010CC.
     self at: #ADTA_GRegs             put: 16r800010CD.
     self at: #ADTA_ColorTable        put: 16r800010CE.
     self at: #ADTA_ColorTable2       put: 16r800010CF.
     self at: #ADTA_Allocated         put: 16r800010D0.
     self at: #ADTA_NumColors         put: 16r800010D1.
     self at: #ADTA_NumAlloc          put: 16r800010D2.

     "Remap animation (defaults to TRUE):"
     self at: #ADTA_Remap             put: 16r800010D3.

     "Screen to remap to:"
     self at: #ADTA_Screen            put: 16r800010D4.

     self at: #ADTA_Width             put: 16r80001259.
     self at: #ADTA_Height            put: 16r8000125A.
     self at: #ADTA_Depth             put: 16r8000125B.

     "Number of frames in the animation:"
     self at: #ADTA_Frames            put: 16r8000125C.

     "Current frame:"
     self at: #ADTA_Frame             put: 16r8000125D.

     "Frames per second:"
     self at: #ADTA_FramesPerSecond   put: 16r8000125E.

     "Amount to change frame by when fast forwarding or
     * rewinding.  Defaults to 10:
     "
     self at: #ADTA_FrameIncrement    put: 16r8000125F.

     "Number of frames to preload; defaults to 10:"
     self at: #ADTA_PreloadFrameCount put: 16r80001260. "(V44)"

     "Sound attributes:"
     self at: #ADTA_Sample            put: 16r800011F6.
     self at: #ADTA_SampleLength      put: 16r800011F7.
     self at: #ADTA_Period            put: 16r800011F8.
     self at: #ADTA_Volume            put: 16r800011F9.
     self at: #ADTA_Cycles            put: 16r800011FA.

     self at: #ADTA_LeftSample        put: 16r80001201. "(V44)"
     self at: #ADTA_RightSample       put: 16r80001202. "(V44)"
     self at: #ADTA_SamplesPerSec     put: 16r800011FF. "(V44)"

     self at: #ADTM_Dummy                put: 16r700.

     "Used to load a frame of the animation:"
     self at: #ADTM_LOADFRAME            put: 16r701.

     "Used to unload a frame of the animation:"
     self at: #ADTM_UNLOADFRAME          put: 16r702.

     "Used to start the animation:"
     self at: #ADTM_START                put: 16r703.

     "Used to pause the animation (don't reset the timer):"
     self at: #ADTM_PAUSE                put: 16r704.

     "Used to stop the animation:"
     self at: #ADTM_STOP                 put: 16r705.

     "Used to locate a frame in the anim' (as set by a slider):"
     self at: #ADTM_LOCATE               put: 16r706.

     "Used to load a new format frame of the animation (V44):"
     self at: #ADTM_LOADNEWFORMATFRAME   put: 16r707.

     "Used to unload a new format frame of the animation (V44):"
     self at: #ADTM_UNLOADNEWFORMATFRAME put: 16r708.
|
   privateSetup
     (uniqueInstance isNil)
       ifTrue: [uniqueInstance <- self privateNew.

                self privateInitializeDictionary
               ].
               
     ^ self    "or ^ uniqueInstance??"
]
