" ------------------------------------------------------------------- "
" PictureTags Class is a Singleton class that allows the user         "
" to reference Picture DataType tags' hexadecimal values.             "
""
" The User does NOT need to create one of these, since DataTypeSystem "
" Class will instantiate the only needed instance of this Class.  See "
" the DataTypeSystem.st source file for the method(s) that help the   "
" User with this Class.                                               "
""
" ALL singleton classes MUST contain the following:                   "
""
"   the methods:  isSingleton AND privateSetup     AND                "
"                 uniqueInstance Class instance variable.             "
" ------------------------------------------------------------------- "

Class PictureTags :Dictionary ! uniqueInstance !
[
   isSingleton
     ^ true  
|  
   privateNew ! newinstance !
     newinstance <- super new.

     ^ newinstance
|
   new
     ^ self privateSetup
|
   privateInitializeDictionary
     "Picture attributes:"

     "Mode ID of the picture:"
     self at: #PDTA_ModeID         put: 16r800010C8.

     "Bitmap header information:"
     self at: #PDTA_BitMapHeader   put: 16r800010C9.

     "Pointer to a class-allocated bitmap, that will end
     * up being freed by picture.class when DisposeDTObject()
     * is called.
     "
     self at: #PDTA_BitMap         put: 16r800010CA.

     "Picture colour table:"
     self at: #PDTA_ColorRegisters put: 16r800010CB.

     "Color table to use with SetRGB32CM():"
     self at: #PDTA_CRegs          put: 16r800010CC.

     "Color table; this table is initialized during the layout
     * process and will contain the colours the picture will use
     * after remapping.  If no remapping takes place, these 
     * colours will match those in the PDTA_CRegs table:
     "
     self at: #PDTA_GRegs          put: 16r800010CD.

     "Shared pen table; this table is initialized during the layout
     * process while the picture is being remapped.
     "
     self at: #PDTA_ColorTable     put: 16r800010CE.

     "Shared pen table; in most places this table will be 
     * identical to the PDTA_ColorTable table.  Some of the
     * colours in this table might match the original colour 
     * palette a little better than the colours picked for 
     * the other table.  The picture.datatype uses the two 
     * tables during remapping, alternating for each pixel.
     "
     self at: #PDTA_ColorTable2    put: 16r800010CF.

     "OBSOLETE; DO NOT USE:"
     self at: #PDTA_Allocated      put: 16r800010D0.

     "Number of colors used by the picture:"
     self at: #PDTA_NumColors      put: 16r800010D1.

     "Number of colors allocated by the picture:"
     self at: #PDTA_NumAlloc       put: 16r800010D2.

     "Remap the picture (BOOL); defaults to TRUE:"
     self at: #PDTA_Remap          put: 16r800010D3.

     "Screen to remap to:"
     self at: #PDTA_Screen         put: 16r800010D4.

     "Free the source bitmap after remapping:"
     self at: #PDTA_FreeSourceBitMap put: 16r800010D5.

     "Pointer to a Point structure:"
     self at: #PDTA_Grab             put: 16r800010D6.

     "Pointer to the destination (remapped) bitmap:"
     self at: #PDTA_DestBitMap       put: 16r800010D7.

     "Pointer to class-allocated bitmap, that will end
     * up being freed by the class after DisposeDTObject()
     * is called:
     "
     self at: #PDTA_ClassBitMap      put: 16r800010D8.

     "Number of colors used for sparse remapping:"
     self at: #PDTA_NumSparse        put: 16r800010D9.

     "Pointer to a table of pen numbers indicating
     * which colors should be used when remapping the image.
     * This array must contain as many entries as there
     * are colors specified with PDTA_NumSparse.
     "
     self at: #PDTA_SparseTable      put: 16r800010DA.

     "Index number of the picture to load. (V44):"
     self at: #PDTA_WhichPicture     put: 16r800010DB.

     "Get the number of pictures stored in the file. (V44):"
     self at: #PDTA_GetNumPictures   put: 16r800010DC.

     "Maximum number of colours to use for dithering. (V44):"
     self at: #PDTA_MaxDitherPens    put: 16r800010DD.

     "Quality of the dithering algorithm to be used during
     * colour quantization. (V44)
     "
     self at: #PDTA_DitherQuality    put: 16r800010DE.

     "Pointer to the allocated pen table. (V44):"
     self at: #PDTA_AllocatedPens    put: 16r800010DF.

     "When querying the # of pictures stored in a file, the
     * following value denotes the # of pictures is unknown
     "
     self at: #PDTANUMPICTURES_Unknown put: 0.

     "V43 extensions (attributes):"

     "Set the sub datatype interface mode:"
     self at: #PDTA_SourceMode         put: 16r800010FA.

     "Set the app datatype interface mode:"
     self at: #PDTA_DestMode           put: 16r800010FB.

     "Allocates the resulting bitmap as a friend bitmap:"
     self at: #PDTA_UseFriendBitMap    put: 16r800010FF.

     "NULL or mask plane for use with BltMaskBitMapRastPort():"
     self at: #PDTA_MaskPlane          put: 16r80001102.

     "Interface modes:"
     self at: #PMODE_V42               put: 0. "Compatibility mode"
     self at: #PMODE_V43               put: 1. "Extended mode"

     "V43 extensions (methods):"
     self at: #PDTM_Dummy              put: 16r660.

     "Transfer pixel data to the picture object in 
     * the specified format
     "
     self at: #PDTM_WRITEPIXELARRAY put: 16r660.

     "Transfer pixel data from the picture object 
     * in the specified format:
     "
     self at: #PDTM_READPIXELARRAY  put: 16r661.

     "Pixel formats:"
     self at: #PBPAFMT_RGB   put: 0. "3 bytes/pixel (rgb)"
     self at: #PBPAFMT_RGBA  put: 1. "4 bytes/pixel (rgb, alpha)"
     self at: #PBPAFMT_ARGB  put: 2. "4 bytes/pixel (alpha, rgb)"
                
     "1 byte/pixel (using a separate colour map):"
     self at: #PBPAFMT_LUT8  put: 3.
                
     "1 byte/pixel (0 is black, 255 is white)"
     self at: #PBPAFMT_GREY8 put: 4.

     "Masking techniques:"
     self at: #mskNone                put: 0.
     self at: #mskHasMask             put: 1.
     self at: #mskHasTransparentColor put: 2.
     self at: #mskLasso               put: 3.
     self at: #mskHasAlpha            put: 4.

     "Compression techniques:"
     self at: #cmpNone                put: 0.
     self at: #cmpByteRun1            put: 1.
     self at: #cmpByteRun2            put: 2. "NOTE: unused (V44)"

     self at: #DTM_WRITEPIXELARRAY    put: 16r660.
     self at: #DTM_READPIXELARRAY     put: 16r661.
     self at: #DTM_CREATEPIXMAPDIR    put: 16r662.
     self at: #DTM_FIRSTPIXMAPDIR     put: 16r663.
     self at: #DTM_NEXTPIXMAPDIR      put: 16r664.
     self at: #DTM_PREVPIXMAPDIR      put: 16r665.
     self at: #DTM_BESTPIXMAPDIR      put: 16r666.

     self at: #PDTA_PixelFormat       put: 16r800010FC.
     self at: #PDTA_TransRemapPen     put: 16r800010FD.
     self at: #PDTA_NumPixMapDir      put: 16r800010FE.
     self at: #PDTA_AlphaChannel      put: 16r80001100.
     self at: #PDTA_MultiRemap        put: 16r80001101.

     "PDTA_SourceMode, PDTA_DestMode:"
     self at: #MODE_V42               put: 0.
     self at: #MODE_V43               put: 1.

|
   privateSetup
     (uniqueInstance isNil)
       ifTrue: [uniqueInstance <- self privateNew.

                self privateInitializeDictionary
               ].
               
     ^ self    "or ^ uniqueInstance??"
]
