" ------------------------------------------------------------------- "
" SoundTags Class is a Singleton class that allows the user           "
" to reference Sound DataType tags' hexadecimal values.               "
""
" The User does NOT need to create one of these, since DataTypeSystem "
" Class will instantiate the only needed instance of this Class.  See "
" the DataTypeSystem.st source file for the method(s) that help the   "
" User with this Class.                                               "
""
" ALL singleton classes MUST contain the following:                   "
""
"   the methods:  isSingleton AND privateSetup     AND                "
"                 uniqueInstance Class instance variable.             "
" ------------------------------------------------------------------- "

Class SoundTags :Dictionary ! uniqueInstance !
[
   isSingleton
     ^ true  
|  
   privateNew ! newinstance !
     newinstance <- super new.

     ^ newinstance
|
   new
     ^ self privateSetup
|
   privateInitializeDictionary

     "Sound attributes:"
     self at: #SDTA_Dummy            put: 16r800011F4.
     self at: #SDTA_VoiceHeader      put: 16r800011F5.

     "Sample data:"
     self at: #SDTA_Sample           put: 16r800011F6.

     "Length of the sample data in UBYTEs:"
     self at: #SDTA_SampleLength     put: 16r800011F7.

     "Period:"
     self at: #SDTA_Period           put: 16r800011F8.

     "Volume. Range from 0 to 64:"
     self at: #SDTA_Volume           put: 16r800011F9.
     self at: #SDTA_Cycles           put: 16r800011FA.

     "The following tags are new for V40:"
     "Task to signal when sound is complete or next buffer 
     * needed.
     "
     self at: #SDTA_SignalTask       put: 16r800011FB.

     "Signal mask to use on completion or 0 to disable
     *
     * NOTE: Due to a bug in sound.datatype V40 SDTA_SignalBit
     *       was actually implemented as a signal mask as 
     *       opposed to a bit number.  The documentation now 
     * reflects this.  If you intend to use a signal bit number
     * instead of the mask, use the new V44 tag
     * SDTA_SignalBitNumber below.
     "
     self at: #SDTA_SignalBit        put: 16r800011FC.
     self at: #SDTA_SignalBitMask    put: 16r800011FC.

     "Playing a continuous stream of data.  Defaults to FALSE."
     self at: #SDTA_Continuous       put: 16r800011FD.

     "The following tags are new for V44:"
     "Signal bit to use on completion or -1 to disable:"
     self at: #SDTA_SignalBitNumber  put: 16r800011FE.

     "Samples per second:"
     self at: #SDTA_SamplesPerSec    put: 16r800011FF.

     "Sample replay period:"
     self at: #SDTA_ReplayPeriod     put: 16r80001200.

     "Sample data:"
     self at: #SDTA_LeftSample       put: 16r80001201.
     self at: #SDTA_RightSample      put: 16r80001202.

     "Stereo panning:"
     self at: #SDTA_Pan              put: 16r80001203.

     "FreeVec all sample data upon OM_DISPOSE:"
     self at: #SDTA_FreeSampleData   put: 16r80001204.

     "Wait for the current sample to be played back before
     * switching to the new sample data:
     "
     self at: #SDTA_SyncSampleChange put: 16r80001205.

     "Data compression methods:"
     self at: #CMP_NONE              put: 0.
     self at: #CMP_FIBDELTA          put: 1.

     "Unity = Fixed 1.0 = maximum volume:"
     self at: #Unity                 put: 16r10000.

     "Channel allocation:"
     self at: #SAMPLETYPE_Left       put: 2.
     self at: #SAMPLETYPE_Right      put: 4.
     self at: #SAMPLETYPE_Stereo     put: 6.
|
   privateSetup
     (uniqueInstance isNil)
       ifTrue: [uniqueInstance <- self privateNew.

                self privateInitializeDictionary
               ].
               
     ^ self    "or ^ uniqueInstance??"
]
