" ------------------------------------------------------------------- "
" DataTypeSystem Class is a Singleton class that allows the user to   "
" reference datatype-specific singleton classes in one spot.          "
""
" ALL singleton classes MUST contain the following:                   "
""
"   the methods:  isSingleton AND privateSetup     AND                "
"                 uniqueInstance Class instance variable.             "
" ------------------------------------------------------------------- "

Class DataTypeSystem :Dictionary 
! uniqueInstance dtClassTags       dtSpecialInfoTags
  dtMethodTags   dtFrameInfoTags   basicDTTags dtToolTags
  dtErrorTags    dtSystemTags      dtAnimTags  dtPictureTags
  dtSoundTags    dtTextTags        idNumbers   iffConstants
!
[
   isSingleton
     ^ true  
|  
   privateNew ! newInstance !
     newInstance <- super new.
     ^ newInstance
|
   new
     ^ (self privateSetup)
|
   privateSetup
     (uniqueInstance isNil)
       ifTrue: [uniqueInstance <- self privateNew.
                
                "Initialize all DataType Singleton classes:"
                dtClassTags       <- DataTypesClassTags new.
                dtSpecialInfoTags <- DTSpecialInfoTags  new.
                dtMethodTags      <- DTMethodTags       new.
                dtFrameInfoTags   <- DTFrameInfoTags    new.
                basicDTTags       <- BasicDataTypeTags  new.
                dtToolTags        <- DataTypeToolTags   new.
                dtErrorTags       <- DataTypeErrorTags  new.
                dtSystemTags      <- DTSystemTags       new.
                dtAnimTags        <- AnimationTags      new.
                dtPictureTags     <- PictureTags        new.
                dtSoundTags       <- SoundTags          new.
                dtTextTags        <- TextTags           new.
                idNumbers         <- IDNumbers          new.
                iffConstants      <- IFFConstants       new
               ].

     ^ self "uniqueInstance??"
|
   getDTClassTag:  key
     ^ dtClassTags at: key
|
   getDTSpecialInfoTag:  key
     ^ dtSpecialInfoTags at: key
|
   getDTMethodTag:  key
     ^ dtMethodTags at: key
|
   getDTFrameInfoTag:  key
     ^ dtFrameInfoTags at: key
|
   getBasicDTTag:  key
     ^ basicDTTags at: key
|
   getDTToolTag:  key
     ^ dtToolTags at: key
|
   getDTErrorTag:  key
     ^ dtErrorTags at: key
|
   getDTSystemTag:  key
     ^ dtSystemTags at: key
|
   getDTAnimationTag:  key
     ^ dtAnimTags at: key
|
   getDTPictureTag:  key
     ^ dtPictureTags at: key
|
   getDTSoundTag:  key
     ^ dtSoundTags at: key
|
   getDTTextTag:  key
     ^ dtTextTags at: key
|
   getIDNumber:  key      "idNumbers are IFF ID codes."
     ^ idNumbers at: key
|
   getIFFConstant: key
     ^ iffConstants at: key
]
