" -------------------------------------------------------------------- "
" ImageTags Class is a Singleton class that allows the user to         "
" reference BOOPSI Image-class tags' hexadecimal values.               "
""
" The User does NOT need to create one of these, since Intuition Class "
" will instantiate the only needed instance of this Class.  See the    "
" SetupIntuition.st source file for the method(s) that help the User   "
" with this Class.                                                     "
""
" ALL singleton classes MUST contain the following:                    "
""
"   the methods:  isSingleton AND privateSetup     AND                 "
"                 uniqueInstance Class instance variable.              "
" -------------------------------------------------------------------- "

Class ImageTags :Dictionary ! uniqueInstance !
[
   isSingleton
     ^ true  
|  
   privateNew ! newinstance !
     newinstance <- super new.

     ^ newinstance
|
   new
     ^ self privateSetup
|
   privateInitializeDictionary
     self at: #IA_Dummy            put: 16r80020000.
     self at: #IA_Left             put: 16r80020001.
     self at: #IA_Top              put: 16r80020002.
     self at: #IA_Width            put: 16r80020003.
     self at: #IA_Height           put: 16r80020004.
     self at: #IA_FGPen            put: 16r80020005.
     self at: #IA_BGPen            put: 16r80020006.
     self at: #IA_Data             put: 16r80020007.
     self at: #IA_LineWidth        put: 16r80020008.
     self at: #IA_ShadowPen        put: 16r80020009.
     self at: #IA_HighlightPen     put: 16r8002000A.

     self at: #IA_Pens             put: 16r8002000E.
     self at: #IA_Resolution       put: 16r8002000F.
     self at: #IA_APattern         put: 16r80020010.
     self at: #IA_APatSize         put: 16r80020011.
     self at: #IA_Mode             put: 16r80020012.
     self at: #IA_Font             put: 16r80020013.
     self at: #IA_Outline          put: 16r80020014.
     self at: #IA_Recessed         put: 16r80020015.
     self at: #IA_DoubleEmboss     put: 16r80020016.
     self at: #IA_EdgesOnly        put: 16r80020017.

     "sysiclass attributes:"
     self at: #SYSIA_Size          put: 16r8002000B.

     "This is unused by Intuition (use SYSIA_DrawInfo):"
     self at: #SYSIA_Depth         put: 16r8002000C.

     self at: #SYSIA_Which         put: 16r8002000D.
     self at: #SYSIA_DrawInfo      put: 16r80020018.

     "New for V39:"
     self at: #SYSIA_ReferenceFont put: 16r80020019.
     self at: #IA_SupportsDisable  put: 16r8002001A.

     self at: #IA_FrameType        put: 16r8002001B.
     self at: #IA_Underscore       put: 16r8002001C.
     self at: #IA_Scalable         put: 16r8002001D.
     self at: #IA_ActivateKey      put: 16r8002001E.
     self at: #IA_Screen           put: 16r8002001F.
     self at: #IA_Precision        put: 16r80020020.

     "data values for SYSIA_Size:"
     self at: #SYSISIZE_MEDRES     put: 0.
     self at: #SYSISIZE_LOWRES     put: 1.
     self at: #SYSISIZE_HIRES      put: 2.

     "SYSIA_Which tag data values.
     * Specifies which system gadget you want an image for.
     * Some numbers correspond to internal Intuition #defines:
     "
     self at: #DEPTHIMAGE  put: 0.  "Window depth gadget image"
     self at: #ZOOMIMAGE   put: 1.  "Window zoom gadget image"
     self at: #SIZEIMAGE   put: 2.  "Window sizing gadget image"
     self at: #CLOSEIMAGE  put: 3.  "Window close gadget image"
     self at: #SDEPTHIMAGE put: 5.  "Screen depth gadget image"
     self at: #LEFTIMAGE   put: 10. "Left-arrow gadget image"
     self at: #UPIMAGE     put: 11. "Up-arrow gadget image"
     self at: #RIGHTIMAGE  put: 12. "Right-arrow gadget image"
     self at: #DOWNIMAGE   put: 13. "Down-arrow gadget image"
     self at: #CHECKIMAGE  put: 14. "GadTools checkbox image"
     self at: #MXIMAGE     put: 15. "GadTools mu-exclude image"

     "New for V39:"
     self at: #MENUCHECK   put: 16. "Menu checkmark image"
     self at: #AMIGAKEY    put: 17. "Menu Amiga-key image"

     "Data values for IA_FrameType (recognized by FrameIClass)"
     self at: #FRAME_DEFAULT     put: 0.
     self at: #FRAME_BUTTON      put: 1.
     self at: #FRAME_RIDGE       put: 2.
     self at: #FRAME_ICONDROPBOX put: 3.

     "Image message id's:"
     self at: #IM_DRAW              put: 16r202. "draw yourself"
                
     "return TRUE if click hits image:"
     self at: #IM_HITTEST           put: 16r203.
                
     "erase yourself:"
     self at: #IM_ERASE             put: 16r204.
                
     "draw new and erase old:"
     self at: #IM_MOVE              put: 16r205.

     "draw with specified dimensions:"
     self at: #IM_DRAWFRAME         put: 16r206.

     "get frame around some box:"
     self at: #IM_FRAMEBOX          put: 16r207.

     "hittest with dimensions:"
     self at: #IM_HITFRAME          put: 16r208.

     "erase with dimensions:"
     self at: #IM_ERASEFRAME        put: 16r209.
               
     "query image for its info:"
     self at: #IM_DOMAINFRAME       put: 16r20A.

     "Image draw states or styles, for IM_DRAW:"
     self at: #IDS_NORMAL           put: 0.
                
     "for selected gadgets:"
     self at: #IDS_SELECTED         put: 1.

     "for disabled gadgets:"
     self at: #IDS_DISABLED         put: 2.
                
     "for future functionality:"
     self at: #IDS_BUSY             put: 3.
               
     "for future functionality (use IDS_INDETERMINANT):"
     self at: #IDS_INDETERMINATE    put: 4.

     "normal, in inactive window border:"
     self at: #IDS_INACTIVENORMAL   put: 5. 
                
     "selected, in inactive border:"
     self at: #IDS_INACTIVESELECTED put: 6.
                
     "disabled, in inactive border:"
     self at: #IDS_INACTIVEDISABLED put: 7.
                
     "disabled and selected:"
     self at: #IDS_SELECTEDDISABLED put: 8.

     "Oops, please forgive spelling error by jimm"
     self at: #IDS_INDETERMINANT    put: 4.

     "For IM_FRAMEBOX:"
     self at: #FRAMEF_SPECIFY       put: 1.

     "Accepted vales for imp_Which:"

     self at: #IDOMAIN_MINIMUM      put: 0.
     self at: #IDOMAIN_NOMINAL      put: 1.
     self at: #IDOMAIN_MAXIMUM      put: 2.
|
   privateSetup
     (uniqueInstance isNil)
       ifTrue: [uniqueInstance <- self privateNew.

                self privateInitializeDictionary
               ].
               
     ^ self    "or ^ uniqueInstance??"
]
