" ------------------------------------------------------------------- "
" TagList Class is mainly used for Amiga function calls that utilize  "
" Tags.  AmigaTalk converts TagLists internally into what the AmigaOS "
" expects for real Amiga tags.                                        "
" Since TagList is an Array to AmigaTalk, you should realize that     "
" you have to set Tags & Values in pairs.  Example:                   "
""
"     myTags    <- TagList new: 24                                    "
""
"     myTags setTag:      (intuition getWindowTag: #WA_Left) index: 0 "
"     myTags setTagValue: (intuition getWindowTag: #WA_Left) value: 16"
""
"     myTags setTag:      (intuition getWindowTag: #WA_Top)  index: 2 "
"     myTags setTagValue: (intuition getWindowTag: #WA_Top)  value: 0 "
"     ...                                                             "
"     myTags setTag:      (intuition specialTag: #TAG_DONE) index: 24 "
""
" Until myTags is filled.                                             "
" WARNING:  Do NOT forget to terminate your TagList with              "
"           (intuition specialTag: #TAG_DONE)                         "
" ------------------------------------------------------------------- "

Class TagList :Array
[
   new: newSize
     ^ (super new: newSize)
|
   getTagValue: theTag
     ^ <primitive 210 33 self theTag>
|
   setTagValue: theTag value: newTagValue
     <primitive 210 32 self theTag newTagValue>
|
   addTagPair: newTag value: newTagValue   "expensive, memory fragmenter"
     ^ <primitive 210 34 self newTag newTagValue>
|
   deleteTagPair: theTag
     ^ <primitive 210 35 self theTag>
|
   setTag: newTag index: idx
     ^ (super at: idx put: newTag)
]

" ------------------------------------------------------------------- "
" SpecialTags Class is a Singleton class that allows the user to      "
" reference Special Tags without having to remember their actual      "
" hexadecimal values.                                                 "
""
" The User does NOT need to create one of these, since Intuition Class"
" will instantiate the only needed instance of this Class.  See the   "
" SetupIntuition.st source file for the method(s) that help the       "
" User with this Class.                                               "
""
" ALL singleton classes MUST contain the following:                   "
""
"   the methods:  isSingleton AND privateSetup     AND                "
"                 uniqueInstance Class instance variable.             "
" ------------------------------------------------------------------- "

Class SpecialTags :Dictionary ! uniqueInstance !
[
   isSingleton
     ^ true  
|  
   privateNew ! newinstance !
     newinstance <- super new.

     ^ newinstance
|
   new
     ^ self privateSetup
|
   privateSetup
     (uniqueInstance isNil)
       ifTrue: [uniqueInstance <- self privateNew.
                
                "terminates array of TagItems. ti_Data unused"
                self at: #TAG_DONE  put: 0.

                self at: #TAG_END   put: 0. "synonym for TAG_DONE"

                "ignore this item, not end of array"
                self at: #TAG_IGNORE put: 1.

                "ti_Data is pointer to another array of TagItems.  
                 Note that this tag terminates the current array
                "
                self at: #TAG_MORE   put: 2.

                "skip this and the next ti_Data items"
                self at: #TAG_SKIP   put: 3.

                "differentiates user tags from control tags:"

                self at: #TAG_USER   put: 16r80000000.

                "If the TAG_USER bit is set in a tag number, it tells 
                 utility.library that the tag is not a control tag 
                 (like TAG_DONE, TAG_IGNORE, TAG_MORE) and is instead 
                 an application tag.  'USER' means a client of 
                 utility.library in general, including system code like 
                 Intuition or ASL, it has nothing to do with user code.
                "
                "Tag filter logic specifiers for use with FilterTagItems()"

                "exclude everything but filter hits"
                self at: #TAGFILTER_AND put: 0.
                self at: #TAGFILTER_NOT put: 1. "exclude only filter hits"

                "Mapping types for use with MapTags()"

                "remove tags that aren't in mapList"
                self at: #MAP_REMOVE_NOT_FOUND put: 0.
                
                "keep tags that aren't in mapList"
                self at: #MAP_KEEP_NOT_FOUND   put: 1.

               ].
               
     ^ self "or ^ uniqueInstance??"
]

