/**
 * Scout - The Amiga System Monitor
 *
 *------------------------------------------------------------------
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * You must not use this source code to gain profit of any kind!
 *
 *------------------------------------------------------------------
 *
 * @author Andreas Gelhausen
 * @author Richard Krber <rkoerber@gmx.de>
 */



#include "system_headers.h"

struct LongFlag wflg_flags[] = {
   {WFLG_SIZEGADGET,    "WFLG_SIZEGADGET",    "($00000001)"},
   {WFLG_DRAGBAR,       "WFLG_DRAGBAR",       "($00000002)"},
   {WFLG_DEPTHGADGET,   "WFLG_DEPTHGADGET",   "($00000004)"},
   {WFLG_CLOSEGADGET,   "WFLG_CLOSEGADGET",   "($00000008)"},
   {WFLG_SIZEBRIGHT,    "WFLG_SIZEBRIGHT",    "($00000010)"},
   {WFLG_SIZEBBOTTOM,   "WFLG_SIZEBBOTTOM",   "($00000020)"},
   {WFLG_BACKDROP,      "WFLG_BACKDROP",      "($00000100)"},
   {WFLG_REPORTMOUSE,   "WFLG_REPORTMOUSE",   "($00000200)"},
   {WFLG_GIMMEZEROZERO, "WFLG_GIMMEZEROZERO", "($00000400)"},
   {WFLG_BORDERLESS,    "WFLG_BORDERLESS",    "($00000800)"},
   {WFLG_ACTIVATE,      "WFLG_ACTIVATE",      "($00001000)"},
   {WFLG_WINDOWACTIVE,  "WFLG_WINDOWACTIVE",  "($00002000)"},
   {WFLG_INREQUEST,     "WFLG_INREQUEST",     "($00004000)"},
   {WFLG_MENUSTATE,     "WFLG_MENUSTATE",     "($00008000)"},
   {WFLG_RMBTRAP,       "WFLG_RMBTRAP",       "($00010000)"},
   {WFLG_NOCAREREFRESH, "WFLG_NOCAREREFRESH", "($00020000)"},
   {WFLG_NEWLOOKMENUS,  "WFLG_NEWLOOKMENUS",  "($00200000)"},
   {WFLG_NW_EXTENDED,   "WFLG_NW_EXTENDED",   "($00040000)"},
   {WFLG_WINDOWREFRESH, "WFLG_WINDOWREFRESH", "($01000000)"},
   {WFLG_WBENCHWINDOW,  "WFLG_WBENCHWINDOW",  "($02000000)"},
   {WFLG_WINDOWTICKED,  "WFLG_WINDOWTICKED",  "($04000000)"},
   {WFLG_VISITOR,       "WFLG_VISITOR",       "($08000000)"},
   {WFLG_ZOOMED,        "WFLG_ZOOMED",        "($10000000)"},
   {WFLG_HASZOOM,       "WFLG_HASZOOM",       "($20000000)"},
   {0, NULL, NULL}};

struct MaskedLongFlag wflg_flags2[] = {
   {WFLG_SMART_REFRESH,  WFLG_REFRESHBITS, "WFLG_SMART_REFRESH",  "($00000000)"},
   {WFLG_SIMPLE_REFRESH, WFLG_REFRESHBITS, "WFLG_SIMPLE_REFRESH", "($00000040)"},
   {WFLG_SUPER_BITMAP,   WFLG_REFRESHBITS, "WFLG_SUPER_BITMAP",   "($00000080)"},
   {WFLG_OTHER_REFRESH,  WFLG_REFRESHBITS, "WFLG_OTHER_REFRESH",  "($000000C0)"},
   {0, 0, NULL, NULL}};

struct LongFlag idcmp_flags[] = {
   {IDCMP_SIZEVERIFY,     "IDCMP_SIZEVERIFY",     "($00000001)"},
   {IDCMP_NEWSIZE,        "IDCMP_NEWSIZE",        "($00000002)"},
   {IDCMP_REFRESHWINDOW,  "IDCMP_REFRESHWINDOW",  "($00000004)"},
   {IDCMP_MOUSEBUTTONS,   "IDCMP_MOUSEBUTTONS",   "($00000008)"},
   {IDCMP_MOUSEMOVE,      "IDCMP_MOUSEMOVE",      "($00000010)"},
   {IDCMP_GADGETDOWN,     "IDCMP_GADGETDOWN",     "($00000020)"},
   {IDCMP_GADGETUP,       "IDCMP_GADGETUP",       "($00000040)"},
   {IDCMP_REQSET,         "IDCMP_REQSET",         "($00000080)"},
   {IDCMP_MENUPICK,       "IDCMP_MENUPICK",       "($00000100)"},
   {IDCMP_CLOSEWINDOW,    "IDCMP_CLOSEWINDOW",    "($00000200)"},
   {IDCMP_RAWKEY,         "IDCMP_RAWKEY",         "($00000400)"},
   {IDCMP_REQVERIFY,      "IDCMP_REQVERIFY",      "($00000800)"},
   {IDCMP_REQCLEAR,       "IDCMP_REQCLEAR",       "($00001000)"},
   {IDCMP_MENUVERIFY,     "IDCMP_MENUVERIFY",     "($00002000)"},
   {IDCMP_NEWPREFS,       "IDCMP_NEWPREFS",       "($00004000)"},
   {IDCMP_DISKINSERTED,   "IDCMP_DISKINSERTED",   "($00008000)"},
   {IDCMP_DISKREMOVED,    "IDCMP_DISKREMOVED",    "($00010000)"},
   {IDCMP_WBENCHMESSAGE,  "IDCMP_WBENCHMESSAGE",  "($00020000)"},
   {IDCMP_ACTIVEWINDOW,   "IDCMP_ACTIVEWINDOW",   "($00040000)"},
   {IDCMP_INACTIVEWINDOW, "IDCMP_INACTIVEWINDOW", "($00080000)"},
   {IDCMP_DELTAMOVE,      "IDCMP_DELTAMOVE",      "($00100000)"},
   {IDCMP_VANILLAKEY,     "IDCMP_VANILLAKEY",     "($00200000)"},
   {IDCMP_INTUITICKS,     "IDCMP_INTUITICKS",     "($00400000)"},
   {IDCMP_IDCMPUPDATE,    "IDCMP_IDCMPUPDATE",    "($00800000)"},
   {IDCMP_MENUHELP,       "IDCMP_MENUHELP",       "($01000000)"},
   {IDCMP_CHANGEWINDOW,   "IDCMP_CHANGEWINDOW",   "($02000000)"},
   {IDCMP_GADGETHELP,     "IDCMP_GADGETHELP",     "($04000000)"},
   {IDCMP_LONELYMESSAGE,  "IDCMP_LONELYMESSAGE",  "($80000000)"},
   {0, NULL, NULL}};

struct LongFlag rtf_flags[] = {
   {RTF_COLDSTART,  "RTF_COLDSTART",  "(1<<0)"},
   {RTF_AFTERDOS,   "RTF_AFTERDOS",   "(1<<2)"},
   {RTF_SINGLETASK, "RTF_SINGLETASK", "(1<<1)"},
   {RTF_AUTOINIT,   "RTF_AUTOINIT",   "(1<<7)"},
   {0, NULL, NULL}};

struct LongFlag pr_flags[] = {
   {PRF_FREESEGLIST, "PRF_FREESEGLIST", "(1)"},
   {PRF_FREECURRDIR, "PRF_FREECURRDIR", "(2)"},
   {PRF_FREECLI,     "PRF_FREECLI",     "(4)"},
   {PRF_CLOSEINPUT,  "PRF_CLOSEINPUT",  "(8)"},
   {PRF_CLOSEOUTPUT, "PRF_CLOSEOUTPUT", "(16)"},
   {PRF_FREEARGS,    "PRF_FREEARGS",    "(32)"},
   {0, NULL, NULL}};

struct LongFlag scr_flags[] = {
   {SHOWTITLE,    "SHOWTITLE",    "($0010)"},
   {BEEPING,      "BEEPING",      "($0020)"},
   {CUSTOMBITMAP, "CUSTOMBITMAP", "($0040)"},
   {SCREENBEHIND, "SCREENBEHIND", "($0080)"},
   {SCREENQUIET,  "SCREENQUIET",  "($0100)"},
   {SCREENHIRES,  "SCREENHIRES",  "($0200)"},
   {PENSHARED,    "PENSHARED",    "($0400)"},
   {NS_EXTENDED,  "NS_EXTENDED",  "($1000)"},
   {AUTOSCROLL,   "AUTOSCROLL",   "($4000)"},
   {0, NULL, NULL}};

struct MaskedLongFlag scr_flags2[] = {
   {WBENCHSCREEN, SCREENTYPE, "WBENCHSCREEN", "($0001)"},
   {PUBLICSCREEN, SCREENTYPE, "PUBLICSCREEN", "($0002)"},
   {CUSTOMSCREEN, SCREENTYPE, "CUSTOMSCREEN", "($000F)"},
   {0, 0, NULL, NULL}};

struct LongFlag lib_flags[] = {
   {LIBF_SUMMING, "LIBF_SUMMING", "(1<<0)"},
   {LIBF_CHANGED, "LIBF_CHANGED", "(1<<1)"},
   {LIBF_SUMUSED, "LIBF_SUMUSED", "(1<<2)"},
   {LIBF_DELEXP,  "LIBF_DELEXP",  "(1<<3)"},
   {0, NULL, NULL}};

struct LongFlag tc_flags[] = {
   {TF_PROCTIME,  "TF_PROCTIME", "(1L<<0)"},
   {TF_ETASK,     "TF_ETASK",    "(1L<<3)"},
   {TF_STACKCHK,  "TF_STACKCHK", "(1L<<4)"},
   {TF_EXCEPT,    "TF_EXCEPT",   "(1L<<5)"},
   {TF_SWITCH,    "TF_SWITCH",   "(1L<<6)"},
   {TF_LAUNCH,    "TF_LAUNCH",   "(1L<<7)"},
   {0, NULL, NULL}};

struct LongFlag psn_flags[] = {
   {PSNF_PRIVATE, "PSNF_PRIVATE", "(0x0001)"},
   {0, NULL, NULL}};

struct MaskedLongFlag mp_flags[] = {
   {PA_SIGNAL,  PF_ACTION, "PA_SIGNAL",  "(0)"},
   {PA_SOFTINT, PF_ACTION, "PA_SOFTINT", "(1)"},
   {PA_IGNORE,  PF_ACTION, "PA_IGNORE",  "(2)"},
   {0, 0, NULL, NULL}};

struct LongFlag bc_flags[] = {
   {1,             "<unknown>",     "(%00000001)"},
   {COF_ACTIVE,    "COF_ACTIVE",    "(%00000010)"},
   {COF_SHOW_HIDE, "COF_SHOW_HIDE", "(%00000100)"},
   {0, NULL, NULL}};

struct LongFlag dip_flags[] = {
   {DIPF_IS_LACE              , "DIPF_IS_LACE"              , "($00000001)"},
   {DIPF_IS_DUALPF            , "DIPF_IS_DUALPF"            , "($00000002)"},
   {DIPF_IS_PF2PRI            , "DIPF_IS_PF2PRI"            , "($00000004)"},
   {DIPF_IS_HAM               , "DIPF_IS_HAM"               , "($00000008)"},
   {DIPF_IS_ECS               , "DIPF_IS_ECS"               , "($00000010)"},
   {DIPF_IS_PAL               , "DIPF_IS_PAL"               , "($00000020)"},
   {DIPF_IS_SPRITES           , "DIPF_IS_SPRITES"           , "($00000040)"},
   {DIPF_IS_GENLOCK           , "DIPF_IS_GENLOCK"           , "($00000080)"},
   {DIPF_IS_WB                , "DIPF_IS_WB"                , "($00000100)"},
   {DIPF_IS_DRAGGABLE         , "DIPF_IS_DRAGGABLE"         , "($00000200)"},
   {DIPF_IS_PANELLED          , "DIPF_IS_PANELLED"          , "($00000400)"},
   {DIPF_IS_BEAMSYNC          , "DIPF_IS_BEAMSYNC"          , "($00000800)"},
   {DIPF_IS_EXTRAHALFBRITE    , "DIPF_IS_EXTRAHALFBRITE"    , "($00001000)"},
   {DIPF_IS_SPRITES_ATT       , "DIPF_IS_SPRITES_ATT"       , "($00002000)"},
   {DIPF_IS_SPRITES_CHNG_RES  , "DIPF_IS_SPRITES_CHNG_RES"  , "($00004000)"},
   {DIPF_IS_SPRITES_BORDER    , "DIPF_IS_SPRITES_BORDER"    , "($00008000)"},
   {DIPF_IS_AA                , "DIPF_IS_AA"                , "($00010000)"},
   {DIPF_IS_SCANDBL           , "DIPF_IS_SCANDBL"           , "($00020000)"},
   {DIPF_IS_SPRITES_CHNG_BASE , "DIPF_IS_SPRITES_CHNG_BASE" , "($00040000)"},
   {DIPF_IS_SPRITES_CHNG_PRI  , "DIPF_IS_SPRITES_CHNG_PRI"  , "($00080000)"},
   {DIPF_IS_DBUFFER           , "DIPF_IS_DBUFFER"           , "($00100000)"},
   {DIPF_IS_PROGBEAM          , "DIPF_IS_PROGBEAM"          , "($00200000)"},
   {DIPF_IS_FOREIGN           , "DIPF_IS_FOREIGN"           , "($80000000)"},
   {0, NULL, NULL}};

struct LongFlag diavail_flags[] = {
   {DI_AVAIL_NOCHIPS          , "DI_AVAIL_NOCHIPS"        , "$0001"},
   {DI_AVAIL_NOMONITOR        , "DI_AVAIL_NOMONITOR"      , "$0002"},
   {DI_AVAIL_NOTWITHGENLOCK   , "DI_AVAIL_NOTWITHGENLOCK" , "$0004"},
   {0, NULL, NULL}};


